/* MiscTreeBrowserCell	-- Cell for MiscTreeBrowser
 *
 */

// IMPORTS

//#import <misckit/MiscTreeBrowserObject.h> // day
#import <misckit/MiscTreeBrowser.h>
#import <misckit/MiscTreeBrowserCell.h>
#import <misckit/MiscTreeBrowserMatrix.h>
#import <misckit/TB_images.h>
//#import <misckit/MiscTreeBrowserProtocols.h> // day


#import <dpsclient/psops.h>
#import <objc/hashtable.h>
#import <appkit/Application.h>
#import <appkit/Text.h>
#import <appkit/NXImage.h>
#import <appkit/Font.h>
#import <stdlib.h>
#import <stdio.h>

// CLASS VARIABLES

static NXCoord indentSize = 10.0;

static NXImage *openImage = nil;
static NXImage *closedImage = nil;
static NXImage *leafImage = nil;

// IMPLEMENTATION

@implementation MiscTreeBrowserCell

// PRIVATE METHODS

+ initialize
{
	if ([MiscTreeBrowserCell class] == self)
	{
//		openImage = [NXImage findImageNamed:"TBOpen"];
//		closedImage = [NXImage findImageNamed:"TBClosed"];
//		leafImage = [NXImage findImageNamed:"TBLeaf"];
		openImage = _MISC_TBOpen_tiff_fn();
		closedImage = _MISC_TBClosed_tiff_fn();
		leafImage = _MISC_TBLeaf_tiff_fn();
	}

	return self;
}

- edit:(const NXRect *)aRect inView:aView editor:textObject delegate:anObject
 event:(NXEvent *)theEvent
{
	return [super edit:aRect
				  inView:aView
				  editor:textObject
				  delegate:anObject
				  event:theEvent];
}


// PUBLIC METHODS

+ (BOOL)prefersTrackingUntilMouseUp
/*"
Returns YES (allows drags outside of Cell) to enable dragging of Cells inside matrix.
"*/
{
	return YES;
}

- init
{
	[super init];

	//  contents = "";
	[self setType:NX_TEXTCELL];
	[self setEditable:NO];

	return self;
}

- setDoubleAction:(SEL)anAction
{
	doubleAction = anAction;

	return self;
}

- (SEL)doubleAction
{
	return doubleAction;
}

- node
{
  return node;
}

- setNode:(id <MiscTreeBrowserNodes>)anObject
{
	indentLevel = 0;
	node = anObject;
	while ((anObject = [anObject parent]))
	{
		indentLevel++;
	}

	return self;
}

- (const char *)stringValue
{
	return [node stringValue];
}

- setStringValue:(const char *)aString
{
	[node setStringValue:aString];

	return self;
}

- (BOOL)isOpen
{
	return (BOOL)hcFlags.open;
}

- (BOOL)isLeaf
{
	return ([node childCount] > 0) ? NO : YES;
}

- setOpen:(BOOL)flag
{
	hcFlags.open = flag;

	return self;
}

- (BOOL)isEditing
{
	return (BOOL)hcFlags.editing;
}

- setEditing:(BOOL)flag
{
	hcFlags.editing = flag;

	return self;
}

/*"
Returns opener image; this can be open, leaf or closed.
"*/
- openerImage
{
	return ([node childCount] == 0)
		? leafImage
		: (hcFlags.open) ? openImage : closedImage;
}

- getOpenerRect:(NXRect *)cellFrame
{
    NXSize imageSize;

	[[self openerImage] getSize:&imageSize];

	cellFrame->origin.x += (indentSize * (float)indentLevel);
	cellFrame->origin.y = NX_Y(cellFrame) + NX_HEIGHT(cellFrame) -
		(NX_HEIGHT(cellFrame) - imageSize.height) / 2.0;
	cellFrame->size = imageSize;

	return self;
}

- getTitleRect:(NXRect *)cellFrame
{
    NXSize imageSize;
	NXCoord	ascender, descender, lineHeight;
	NXCoord xOffset, yOffset;

	NXTextFontInfo(support, &ascender, &descender, &lineHeight);
	[[self openerImage] getSize:&imageSize];
	xOffset = (indentSize * (float)indentLevel) + imageSize.width + 2.0;
	yOffset = (cellFrame->size.height - lineHeight +0.5) / 2.0;

	cellFrame->origin.x += xOffset;
	cellFrame->origin.y +=  yOffset;
	cellFrame->size.height = lineHeight;
	cellFrame->size.width -= xOffset;

	return self;
}

- drawInside:(const NXRect *)cellFrame inView:controlView
{
	NXRect rect;
	NXCoord	ascender, descender, lineHeight;
	NXImage *image;
	NXCoord	baseX, baseY;
	const char *stringValue = [self stringValue];

								// get text info; should use shared textcell!
	NXTextFontInfo(support, &ascender, &descender, &lineHeight);
	baseX = NX_X(cellFrame);
	baseY = NX_Y(cellFrame) + lineHeight - descender;

								// draw cell background
	if (cFlags1.highlighted)
	{
		NXRect rect = *cellFrame;
		NXCoord height = rect.size.height;
								// white background highlights cell
		PSsetgray(NX_WHITE);
		NXRectFill(cellFrame);

		if (!hcFlags.editing)
		{						// if not editing, grey borders above'n'below
			PSsetgray(NX_DKGRAY);
			rect.size.height = 1.0;
			NXRectFill(&rect);
			rect.origin.y += (height - 1.0);
			NXRectFill(&rect);
		}
	}
	else
	{
		NXColor backgroundColor = [controlView cellColor];

		PSsetrgbcolor(NXRedComponent(backgroundColor),
					  NXGreenComponent(backgroundColor),
					  NXBlueComponent(backgroundColor));
		NXRectFill(cellFrame);
	}

	PSsetgray(NX_BLACK);

	/* draw open/leaf/closed icon at correct indentation */

	image = [self openerImage];
	rect = *cellFrame;
	[self getOpenerRect:&rect];
	[image composite:NX_SOVER toPoint:&rect.origin];
			
	if (stringValue)
	{
		id textCell = [controlView sharedTextCell];

		rect = *cellFrame;
		[self getTitleRect:&rect];
		[textCell setStringValue:[self stringValue]];
		[textCell setEnabled:[self isEnabled]];
		[textCell drawInside:&rect inView:controlView];
	}

	return self;
}

/*(
Draws border then insides of cell.
)*/
- drawSelf:(const NXRect *)cellFrame inView:aView
{
	return [self drawInside:cellFrame inView:aView];
}

/*(
As drawInside:inView: above, but highlighted
)*/
- highlight:(const NXRect *)cellFrame inView:controlView lit:(BOOL)flag
{
    if (cFlags1.highlighted != flag)
	{
		cFlags1.highlighted = flag;
		[self drawSelf:cellFrame inView:controlView];
    }
    return self;
}

// Dragging methods

- (BOOL)startTrackingAt:(const NXPoint *)startPoint inView:aView
{
	printf("s_t\n");

	return YES;
}

- (BOOL)continueTracking:(const NXPoint *)lastPoint
					  at:(const NXPoint *)currentPoint
				  inView:aView
{
	printf("c_t\n");

	return YES;
}

- stopTracking:(const NXPoint *)lastPoint
			at:(const NXPoint *)stopPoint
		inView:aView
	 mouseIsUp:(BOOL)flag
{
	printf("stop_t\n");

	return self;
}

@end
