/* MiscTreeBrowserMatrix
 *
 */

// IMPORTS

#import <math.h>
#import <appkit/Application.h>
#import <appkit/TextField.h>
#import <appkit/TextFieldCell.h>
#import <appkit/Text.h>

#import <misckit/MiscTreeBrowser.h>
#import <misckit/MiscTreeBrowserCell.h>
#import <misckit/MiscTreeBrowserMatrix.h>
#import <misckit/MiscTreeBrowserProtocols.h>

// DEFINES

#define RETURN	13
#define ESCAPE	27
#define DELETE	127

#define LEFTARROW	172
#define RIGHTARROW	174
#define UPARROW		173
#define DOWNARROW	175

// IMPLEMENTATION

@implementation MiscTreeBrowserMatrix

/*"
Returns our controlling MiscTreeBrowser 
"*/
- _treeBrowser
{
	return [superview superview];
}

/*"
Return the textfield used for editing cells.
"*/
- _editField
{
	if (!editField)
	{
		NXRect aFrame = { 0.0, 0.0, 0.0, 0.0 };

		aFrame.size = cellSize;
		editField = [[TextField alloc] initFrame:&aFrame];
		if (font)
		{
			[editField setFont:font];
		}
		[editField setEditable:YES];
		[editField setBezeled:NO];
		[editField setBordered:NO];
		[editField setTextDelegate:self];
	}

	return editField;
}

/*(
Returns frame of cell that has been clicked
)*/
- _setCellFrame:(NXRect *)cellFrame
  mousePoint:(NXPoint *)mousePt
  forEvent:(NXEvent *)theEvent
{
	id scell;
	NXCoord rowHeight = cellSize.height + intercell.height;
	float rowNum;

	*mousePt = theEvent->location;
	[self convertPoint:mousePt fromView:nil];
	rowNum = floor(mousePt->y / rowHeight);

	cellFrame->origin.x = 0.0;
	cellFrame->origin.y = rowNum * rowHeight;
	cellFrame->size = cellSize;

	scell = [self cellAt:rowNum :0];

	return scell;
}

/*"
"*/
- _toggleCellOpen
{
	id aCell = [self selectedCell];

	if ([aCell isOpen])
	{
		[[self _treeBrowser] closeCell:aCell];
	}
	else
	{
		[[self _treeBrowser] openCell:aCell];
	}
	return self;
}

// PUBLIC METHODS

- initFrame:(const NXRect *)frRect 
	   mode:(int)aMode
  cellClass:classId
	numRows:(int)r
	numCols:(int)c
{
	[super initFrame:frRect
		   mode:aMode			// NX_TRACKMODE for dragging...
		   cellClass:classId
		   numRows:r numCols:c];

	textCell = [[TextFieldCell alloc] init];

	return self;
}

- initFrame:(const NXRect *)frameRect 
	   mode:(int)aMode
  prototype:aCell
	numRows:(int)r
	numCols:(int)c
{
	[super initFrame:frameRect mode:aMode prototype:aCell numRows:r numCols:c];

	textCell = [[TextFieldCell alloc] init];

	return self;
}

/*(
Returns the text cell to be used for writing the text in a cell.
)*/
- sharedTextCell
{
	return textCell;
}

- setFont:aFont
{
	[[self _editField] setFont:aFont];
	[[self sharedTextCell] setFont:aFont];
	[super setFont:aFont];

	return self;
}

- sendDoubleAction
{
	id scell;
	NXRect cellFrame, titleFrame;
	NXPoint mousePt;
	NXEvent *theEvent = [NXApp currentEvent];

	if (editCell) printf("edit + click!\n");

	scell = [self _setCellFrame:&cellFrame
				  mousePoint:&mousePt
				  forEvent:theEvent];

	titleFrame = cellFrame;
	[scell getTitleRect:&titleFrame];

	if ([self mouse:&mousePt inRect:&titleFrame])
	{							// double-click on text; edit it
		[self editCell:scell];

		return self;
	}
	else
	{							// perform double-click action
		SEL cellDoubleAction;

		if ((scell = [self selectedCell]) && 
			[scell respondsTo:@selector(doubleAction)]  &&
			(cellDoubleAction = [scell doubleAction]))
		{
			[[scell target] perform:cellDoubleAction with:self];

			return self;
		}
		else 
		{
			return [super sendDoubleAction];
		}
	}
}

- (NXColor)cellColor
{
	return [[self _treeBrowser] backgroundColor];
}


// EVENT HANDLING

- mouseDragged:(NXEvent *)theEvent
{
	printf("dr\n");

	return self;
}

/*(
Handle mouse clicks and double-clicks in all the various areas of the cell.
)*/
- mouseDown:(NXEvent *)theEvent
{
	id scell;
	NXRect cellFrame;
	NXPoint mousePt;

	scell = [self _setCellFrame:&cellFrame
				  mousePoint:&mousePt
				  forEvent:theEvent];
	if (editCell && (editCell != scell))
	{			// we are already editing a (different) cell, so end editing.
		[self textDidEnd:[[editField window] getFieldEditor:NO for:editField]
				 endChar:NX_RETURN];
	}
	[scell getOpenerRect:&cellFrame];
	cellFrame.origin.y -= cellFrame.size.height; // frame is flipped

	if ([self mouse:&mousePt inRect:&cellFrame])
	{
		[self perform:@selector(_toggleCellOpen)
			  with:nil
			  afterDelay:1
			  cancelPrevious:YES];
	}

	return [super mouseDown:theEvent];
}

/*"
Returns YES -- matrix allows use of arrow keys, Delete (delete cell) and others.
"*/
- (BOOL)acceptsFirstResponder
{
	return YES;
}

/*"
Does nothing, but eliminated "no responder" beeps
"*/
- keyDown:(NXEvent *)theEvent
{
	return self;
}

- keyUp:(NXEvent *)theEvent
{
	unsigned short keypress = theEvent->data.key.charCode;
	int sRow;
	id sCell;

	if (wasEditing)
	{							// ignore keystrokes whilst editing cell
		if (!editCell)
		{						// skip 'Return' KeyUp at end of edit
			wasEditing = NO;
		}
		return self;
	}

	switch (keypress)
	{
	case DELETE:
		[[self _treeBrowser] removeCellAt:[self selectedRow]];
		break;

	case RETURN:
		[[self _treeBrowser] addCellAt:[self selectedRow]];
		break;

	case UPARROW:
		// select next cell up, or last cell if at top or none selected
		sRow = [self selectedRow];
		sRow = ((sRow <= 0) ? numRows : sRow) - 1;
		[self selectCellAt:sRow : 0];
		[self scrollCellToVisible:sRow :0];
		break;

	case DOWNARROW:
		// select next cell down, or first cell if at end or none selected
		sRow = [self selectedRow] + 1;
		sRow = (sRow >= numRows) ?  0 : sRow;
		[self selectCellAt:sRow : 0];
		[self scrollCellToVisible:sRow :0];
		break;

	case LEFTARROW:
		// if cell is open, close it.  If it is not open, and selected cell is
		// not a root cell, select the parent cell
		sCell = [self selectedCell];
		if ([sCell isOpen])
		{
			[self _toggleCellOpen];
		}
		else
		{
			id parent = [[sCell node] parent];

			if (parent)
			{
				sRow = [self selectedRow] - 1;
				while ((sRow > 0) && ([[self cellAt:sRow :0] node] != parent))
				{
					sRow--;
				}
				[self selectCellAt:sRow : 0];
				[self scrollCellToVisible:sRow :0];
			}
		}
		break;

	case RIGHTARROW:
		// if cell is not open, open it.  If it is open and has children,
		// select first child
		sCell = [self selectedCell];
		if ([sCell isOpen])
		{
			if (![sCell isLeaf])
			{
				sRow = [self selectedRow] + 1;
				[self selectCellAt:sRow : 0];
				[self scrollCellToVisible:sRow :0];
			}
		}
		else
		{
			[self _toggleCellOpen];
		}
		break;
	}

	return self;
}

// text (field editor) delegate methods

- textDidEnd:sender endChar:(unsigned short)reason
{
	if (reason != ESCAPE)
	{
		[editCell setStringValue:[editField stringValue]];
	}
	[editField removeFromSuperview];

	[editCell setEditing:NO];
	[self selectCell:editCell];
	editCell = nil;
	[[self window] perform:@selector(makeFirstResponder:)
				   with:self
				   afterDelay:1
				   cancelPrevious:YES];

	return self;
}

- editCell:aCell
{
	if ((aCell == [self selectedCell]) || ([self selectCell:aCell] == aCell))
	{
		int row, col;
		NXRect textFrame;
		id editor = [self _editField];

		editCell = aCell;
		wasEditing = YES;

		[self getRow:&row andCol:&col ofCell:editCell];
		[self getCellFrame:&textFrame at:row :col];
		[editCell getTitleRect:&textFrame];
		[editCell setEditing:YES];
		[self drawCell:editCell];
		[editor setStringValue:[editCell stringValue]];
		[editor setFrame:&textFrame];
		[self addSubview:editor];

		[[self window] makeFirstResponder:editor];
		[editor display];
		[editor selectText:self];

		return self;
	}

	return nil;
}

- (BOOL)isEditing
{
	return editCell ? YES : NO;
}

@end
