// Mon Sep  2 12:09:44 MET DST 1996

#import "HRCell.h"

#import <appkit/appkit.h>

#define CLASS_VERSION 1

@implementation HRCell:Cell

+ initialize
 {   if (self == [HRCell class]) 
	{	/*
		 * **** Archiving: READ ME **** After bumping the CLASS_VERSION, it is
		 * considered common practice to add a comment line indicating the new
		 * version number, date, and modifier. Optionally, the reason for the
		 * change. There is no need to modify the setVersion message.
		 */
		// version 1: initial.  (db)
		[[self class] setVersion:CLASS_VERSION];
    }
	return self;
}


- initForText:(Text*) theText
{	if(!(self=[super init])) return nil;
	text=theText;
	return self;
}

// could be useful for unregistering
- free
{	return [super free];
}

- calcCellSize:(NXSize *)theSize
{	NXRect drawRect;
	[text getFrame:&drawRect];
	theSize->width=drawRect.size.width-12;
	theSize->height = 6;
    return self;
}

- drawInside: (const NXRect *)cellFrame inView:controlView
{	NXRect drawRect,clipRect;
	drawRect=*cellFrame;
	[self calcCellSize:&drawRect.size];
	[controlView getFrame:&clipRect];
	[controlView lockFocus];
	NX_WIDTH(&drawRect)+=(NX_X(&drawRect)+12);NX_X(&drawRect)=0;
	clipRect=drawRect;NX_X(&clipRect)-=1;NX_Y(&clipRect)-=1;
	NX_WIDTH(&clipRect)+=2;	NX_HEIGHT(&clipRect)=3;
	NXDrawGroove(&drawRect,&clipRect);
	[controlView unlockFocus];
	return self;
}

- highlight:(const NXRect *)cellFrame inView:controlView lit:(BOOL)flag;
{	return [self drawInside:cellFrame inView:controlView];
}

#define eventMask NX_MOUSEUPMASK | NX_MOUSEDRAGGEDMASK
#define CLIP_Y(y, restraint0,restraintN) MAX(restraint0,MIN(restraintN,y))

- (BOOL)trackMouse:(NXEvent *)theEvent 
			inRect:(const NXRect *)cellFrame 
			ofView:controlView;
{	return NO;
}


@end


@implementation TFTextFieldCell

- drawSelf: (const NXRect *)cellFrame inView:controlView
{	id erg;
	PSgsave();
	erg=[super drawSelf:cellFrame inView:controlView];
	PSgrestore();
	return erg;
}
- drawInside: (const NXRect *)cellFrame inView:controlView
{	if(afterDrag)
	{	afterDrag=NO;
		return self;
	} else return [super drawInside:cellFrame inView:controlView];
}

- (BOOL)trackMouse:(NXEvent *)theEvent 
			inRect:(const NXRect *)cellFrame 
			ofView:(Text*)controlView;
{	BOOL	erg;
 	NXRect frame=*cellFrame;
	if(  [controlView isKindOf:[Text class]])
	{	[[controlView window] disableFlushWindow];
		[[controlView superview] convertRect:&frame fromView:controlView];
		[[controlView sharedField] setCell:self];
		[[controlView sharedField] removeFromSuperview];
		[[controlView superview] addSubview:[controlView sharedField]];
		[[controlView sharedField] setFrame:&frame];
		[[controlView window] reenableFlushWindow];
		[[controlView sharedField] mouseDown:theEvent];
		afterDrag=YES;
		return NO;
	} else erg=[super trackMouse:theEvent inRect:cellFrame ofView:controlView];
	return erg;
}

-(void) setWidth:(NXCoord)aWidth
{	width=aWidth;
}
- calcCellSize:(NXSize *)theSize
{	[super calcCellSize:theSize];
	if(width) theSize->width=width;
    return self;
}

- highlight:(const NXRect *)cellFrame inView:controlView lit:(BOOL)flag;
{	return [self drawInside:cellFrame inView:controlView];
}

@end

@implementation TFRadioCell

- initInText:(Text*)view
{	self=[self initIconCell:"FormRadio"];
	[self setAltIcon:"FormRadioH"];
	[self setBordered:NO];
	[self setHighlightsBy:NX_CONTENTS];
	[self setShowsStateBy:NX_CONTENTS];

	return self;
}
-(void) setCollegues:(List*) theCollegues
{	collegues=theCollegues;
}
- (BOOL)trackMouse:(NXEvent *)theEvent 
			inRect:(const NXRect *)cellFrame 
			ofView:(Text*)controlView;
{	if([self state]) return NO;
	else	// do some radio
	{	[collegues removeObject:self];
		[collegues makeObjectsPerform:@selector(setState:) with:(id)NO];
		[controlView display];
		[collegues addObject:self];
		return [super trackMouse:theEvent inRect:cellFrame ofView:controlView];
	}
}

@end
@implementation TFCheckCell

- initInText:(Text*)view
{	self=[self initIconCell:"NXswitch"];
	[self setAltIcon:"NXswitch"];
	[self setBordered:NO];
	[self setHighlightsBy:NX_CONTENTS];
	[self setShowsStateBy:NX_CONTENTS];

	return self;
}

@end

@implementation TFPopupCell

- initInText:(Text*)view
{	self=[self initTextCell:""];
	[self setIcon:"NXpopup"];
	[self setIconPosition:NX_ICONRIGHT];
	[self setHighlightsBy:NX_PUSHIN];
	[self setShowsStateBy:NX_PUSHIN];
	[self setAlignment:NX_LEFTALIGNED];
	text=view;

	return self;
}
-(void) setPopupList:(PopUpList*) theList
{	popupList=theList;
}
- takeTitleFrom:sender
{	[self setTitle:[[sender selectedCell] title]];
	[text display];
	return self;
}
- (BOOL)trackMouse:(NXEvent *)theEvent 
			inRect:(const NXRect *)cellFrame 
			ofView:(Text*)controlView;
{	NXRect frame=*cellFrame;
	if([controlView isKindOf:[Text class]])
	{	id newButton=NXCreatePopUpListButton(popupList);
		[[controlView window] disableFlushWindow];
		[[controlView superview] convertRect:&frame fromView:controlView];
	// register so that previous gets removed and freed
		[controlView setSharedButton:newButton];
		[[controlView superview] addSubview:newButton];
		[newButton setFrame:&frame];
		[[controlView window] reenableFlushWindow];
		[newButton mouseDown:theEvent];
		return NO;
	} else return [super trackMouse:theEvent inRect:cellFrame ofView:controlView];
}
-(void) setWidth:(NXCoord)aWidth
{	width=aWidth;
}
- calcCellSize:(NXSize *)theSize
{	[super calcCellSize:theSize];
	if(width) theSize->width=width;
    return self;
}
@end

@implementation TextBackgroundCell

- initFromImage:(NXImage*) theImg
{	if(!theImg) return nil;
	self=[super init];
	image=theImg;
	return self;
}

- calcCellSize:(NXSize *)theSize
{	theSize->width=2;theSize->height=2;
    return self;
}
- drawInside: (const NXRect *)cellFrame inView:controlView
{	NXRect clipRect;
	NXSize imgSiz={0,0};
	NXCoord		x,y;
	
	if(!image) return self;

	[controlView getFrame:&clipRect];

	[image getSize:&imgSiz];
	if(!imgSiz.width || !imgSiz.height) return self;

	[controlView lockFocus];
	for(y=imgSiz.height;y< NX_HEIGHT(&clipRect);y+=imgSiz.height)
	{	for(x=0;x< NX_WIDTH(&clipRect);x+=imgSiz.width)
		{	NXPoint currPoint={x,y};
			[image composite:NX_COPY toPoint:&currPoint];
		}
	} [controlView unlockFocus];
	return self;
}
- highlight:(const NXRect *)cellFrame inView:controlView lit:(BOOL)flag;
{	return nil;
}

@end


// this one displays the table-content cells in HTML-matrices
@implementation HTMLTextCell
static id sharedText;
- sharedText
{	if(!sharedText)
	{	NXRect someFrame={0,0,120,16};
		sharedText=[[Text alloc] initFrame:&someFrame];
		[sharedText setDelegate:delegate];
		[sharedText setVertResizable:NO];
		[sharedText setHorizResizable:YES];
	}
	return sharedText;
}
- sizeToFit
{	NXRect frame;
	if(!strchr([self stringValue],'<')) [[self sharedText] setText:[self stringValue]];
	else								[[self sharedText] setFromHTML:[self stringValue]];
	[[self sharedText] sizeToFit];[[self sharedText] getFrame:&frame];
	size=frame.size;
	return self;
}

- initWithHTML:(const char*) theHTML inView:(Text*) theText
{	[self initTextCell:theHTML];
	delegate=[theText delegate];
	if(![[self sharedText] isDescendantOf:[theText superview]])
		[[theText superview] addSubview:[self sharedText]];
	text=theText;
	return self;
}
- setStringValue:(const char*) theStr
{	id erg=[super setStringValue:theStr];
	[self sizeToFit];
	return erg;
}
- calcCellSize:(NXSize *)theSize
{	*theSize=size;
    return self;
}
- drawInside: (const NXRect *)cellFrame inView:controlView
 {	NXRect frame=*cellFrame;
	[[text superview]	convertRect:&frame fromView:text];
	if(!strchr([self stringValue],'<')) [[self sharedText] setText:[self stringValue]];
	else								[[self sharedText] setFromHTML:[self stringValue]];
	PSgsave();
	[[self sharedText] setFrame:&frame];
	//[[self sharedText] display];
	PSgrestore();
	return NO;
}

- readRichText:(NXStream *)stream forView:view
{	char *buffer;
	int	  buflen;

	NXScanf(stream, "%f %f %d",&size.width,&size.height,&buflen);
	buffer=malloc(buflen+1);
	NXScanf(stream, "%s",buffer);
	[self initWithHTML:buffer inView:view];
	free(buffer);

	return self;
}
- writeRichText:(NXStream *)stream forView:view
{	NXPrintf(stream, "%f %f %d %s",size.width,size.height,strlen([self stringValue]),[self stringValue]);
	return self;
}
-(void) setSize:(NXSize*) aSize {size=*aSize;}
-(void) setText:(Text*) aText {text=aText;}
- copyFromZone:(NXZone*) aZone
{	HTMLTextCell *newCell=[super copyFromZone:aZone];
	[newCell setSize:&size];
	[newCell setStringValue:[self stringValue]];
	[newCell setText:text];
	
	return newCell;
}

@end


@implementation TFMatrixCell

- initWithMatrix:(Matrix *)aMatrix inView:controlView
{	matrix=aMatrix;
	if(![matrix isDescendantOf:[controlView superview]])
		[[controlView superview] addSubview:matrix];
	return self;
}
- drawInside: (const NXRect *)cellFrame inView:controlView
 {	NXRect frame=*cellFrame;
	[[controlView superview] convertRect:&frame fromView:controlView];
	PSgsave();
	[matrix	setFrame:&frame];
	//[matrix	drawSelf:cellFrame:1];
	PSgrestore();
	return NO;
}
- calcCellSize:(NXSize *)theSize
{	NXRect frame;
	[matrix getFrame:&frame];
	*theSize=frame.size;
    return self;
}

- free
{	if(matrix)
	{	[matrix removeFromSuperview];
		[matrix free];
	}
	return [super free];
}

@end
