#import "Text.DExtensios.h"

// toDo		ctrl-Backspace: delete line
//			shiftCrsrUp/dwn: pagewise scrolling


@interface Object(TexDelegateExtensions)
-(BOOL) escapePressed:sender;
@end

int getCharOfText(Text *self,int pos)
{	char text[2];
	if([self getSubstring:text start:pos length:1] != 1) return -1;
	return text[0];
}

@implementation Text(DExtensions)

- setFromRichText:(const char*) theStr ofLength:(int) len
{	NXStream *stream;
	vm_address_t tmpMem;
	if(!len || !theStr)
	{	[window disableFlushWindow];
		[[self selectAll:self] delete:self];
		[window reenableFlushWindow];
		return self;
	}
	vm_allocate(task_self(),&tmpMem,len,TRUE);
	memcpy((void*)tmpMem,theStr,len);
	stream=NXOpenMemory((void*)tmpMem,len,NX_READONLY);
	[window disableFlushWindow];
	[self readRTFDFrom:stream];
	[window reenableFlushWindow];
	NXCloseMemory(stream,NX_FREEBUFFER);
	return self;
}
-(char*) richTextAsCopyOfLength:(int*) theLen
{	NXStream *stream=NXOpenMemory(NULL,0,NX_READWRITE);
	char *streambuf,*mallocbuf;
	int		len, maxlen;
	[self writeRTFDTo:stream];
	NXGetMemoryBuffer(stream, &streambuf,&len,&maxlen);
	mallocbuf=malloc(len);
	memcpy(mallocbuf,streambuf,len);
   *theLen=len;
	NXCloseMemory(stream, NX_FREEBUFFER);
	return mallocbuf;
}
-(char*) textAsCopy
{	int	  theLen=[self textLength];
	char *buf=malloc(theLen+1);
	[self getSubstring:buf start:0 length:theLen+1];
	return buf;
}
-(MiscString*) textAsString
{	char *buf=[self textAsCopy];
	MiscString *theStr=[[MiscString alloc] initString:buf];
	free(buf);
	return theStr;
}
-(char*) selectedTextAsCopy
{	int	  theLen=spN.cp-sp0.cp;
	char *buf=malloc(theLen+1);
	[self getSubstring:buf start:sp0.cp length:theLen];
	buf[theLen]=0;
	return buf;
}
- selectWordUnderCaret
{	int lpos=sp0.cp,rpos=spN.cp,c;
	if(lpos ==rpos)
	{	while((c=getCharOfText(self,rpos))>= 0 && !NXIsSpace(c)) rpos++;
		if(NXIsSpace( getCharOfText(self,lpos))) lpos--;
		while(lpos && (c=getCharOfText(self,lpos))>= 0 && !NXIsSpace(c)) lpos--;
		if(NXIsSpace(c)) lpos++;
		[self setSel:lpos:rpos];
	}
	return self;
}


-(int) lengthOfLineForSel:(NXSelPt *)sel
{	return theBreaks->breaks[sel->line/sizeof(NXLineDesc)]&0x3fff;
}


- scrollHeight:(NXCoord) theHeight
{	if([[self superview] isKindOf:[ClipView class]])
	{	NXRect	theRect;
		NXEvent postEvent = (NXEvent)*[NXApp currentEvent];
		NXPoint thePoint;
		[[self superview] getBounds:&theRect];
		thePoint.x=theRect.origin.x;
		thePoint.y=MAX(theRect.origin.y-theHeight,0);
		[self scrollPoint:&thePoint];
		thePoint.x=sp0.x,thePoint.y=MAX(sp0.y-theHeight,0);
		[self convertPoint:&thePoint toView:nil];
		postEvent.location = thePoint;
		postEvent.type = NX_MOUSEUP,postEvent.flags=0;
		DPSPostEvent(&postEvent, YES);
		postEvent.type = NX_MOUSEDOWN,postEvent.data.mouse.click = 1;
		[self mouseDown:&postEvent];
		return thePoint.y? nil:self;
	} return nil;
}

@end

//	keyFilter stuff

NXCharFilterFunc oldFilter;

void installCharFilter(Text *self,NXCharFilterFunc newFilter)
{	oldFilter=[self charFilter];
	[self setCharFilter:newFilter];
}
unsigned short WriteNowCharfilter(unsigned short charCode,int keyFlags,unsigned short charSet)
{	int pos,c;
	Text	*currText=[[NXApp keyWindow] firstResponder];
	NXSelPt sp0,spN;
	[currText getSel:&sp0:&spN];
	switch(NXEditorFilter(charCode,0,charSet))	// dont set it see the flags (esp.cntrl)
	{	case NX_LEFT:
			if(keyFlags & NX_SHIFTMASK)
			{	if(sp0.cp == sp0.c1st) return 0;
				[currText setSel:sp0.c1st+1:sp0.c1st+1];
			} else if(keyFlags & NX_CONTROLMASK)
			{	pos=sp0.cp;
				while(pos && (c=getCharOfText(currText,--pos))>= 0 && !NXIsSpace(c)){}
				[currText setSel:pos+1:pos+1];
				return NX_LEFT;		// default cntrl-crsr lets arrowos be inserted into text
			}
		break;
		case NX_RIGHT:
			if(keyFlags & NX_SHIFTMASK)
			{	pos=spN.c1st+[currText lengthOfLineForSel:&spN]-1;
				if(pos == spN.cp) return 0;
				[currText setSel:pos-1:pos-1];
			} else if(keyFlags & NX_CONTROLMASK)
			{	pos=spN.cp;
				while((c=getCharOfText(currText,++pos))>= 0 && !NXIsSpace(c)){}
				[currText setSel:pos-1:pos-1];
				return NX_RIGHT;
			}
		break;
		case NX_UP:
			if(keyFlags & NX_SHIFTMASK)		//pageUp
			{	NXRect	theRect;
				[[currText superview] getBounds:&theRect];
				if([currText scrollHeight:theRect.size.height])
					return NX_RIGHT;
			}
		break;
		case NX_DOWN:
			if(keyFlags & NX_SHIFTMASK)		//pageUp
			{	NXRect	theRect;
				[[currText superview] getBounds:&theRect];
				if([currText scrollHeight:-theRect.size.height])
					return NX_LEFT;
			}
		break;
		case NX_DELETE:
			if(keyFlags & NX_CONTROLMASK)	// delete curr. line
			{
			}
		break;
		case 32:	//esc or spc
		{	id delegate=[currText delegate];
			if(charCode == 32 && !(keyFlags & NX_COMMANDMASK)) break;
			if([delegate respondsTo:@selector(escapePressed:)])
			{	if(![delegate escapePressed:currText])
					return 0;
				else keyFlags&=~NX_COMMANDMASK;
			}
		} break;
	} return oldFilter(charCode,keyFlags,charSet);
}
//	end keyFilter stuff

